@@ -47,6 +47,36 @@ enum {
   SILCTXT_CHANNEL_KILLED,
   SILCTXT_CHANNEL_QUIETED,
   SILCTXT_CHANNEL_APPEARS,
+  SILCTXT_PUBMSG_HILIGHT_SIGNED,
+  SILCTXT_PUBMSG_HILIGHT_UNKNOWN,
+  SILCTXT_PUBMSG_HILIGHT_FAILED,
+  SILCTXT_PUBMSG_ME_SIGNED,
+  SILCTXT_PUBMSG_ME_UNKNOWN,
+  SILCTXT_PUBMSG_ME_FAILED,
+  SILCTXT_PUBMSG_SIGNED,
+  SILCTXT_PUBMSG_UNKNOWN,
+  SILCTXT_PUBMSG_FAILED,
+  SILCTXT_PUBMSG_HILIGHT_CHANNEL_SIGNED,
+  SILCTXT_PUBMSG_HILIGHT_CHANNEL_UNKNOWN,
+  SILCTXT_PUBMSG_HILIGHT_CHANNEL_FAILED,
+  SILCTXT_PUBMSG_ME_CHANNEL_SIGNED,
+  SILCTXT_PUBMSG_ME_CHANNEL_UNKNOWN,
+  SILCTXT_PUBMSG_ME_CHANNEL_FAILED,
+  SILCTXT_PUBMSG_CHANNEL_SIGNED,
+  SILCTXT_PUBMSG_CHANNEL_UNKNOWN,
+  SILCTXT_PUBMSG_CHANNEL_FAILED,
+  SILCTXT_MSG_PRIVATE_SIGNED,
+  SILCTXT_MSG_PRIVATE_UNKNOWN,
+  SILCTXT_MSG_PRIVATE_FAILED,
+  SILCTXT_MSG_PRIVATE_QUERY_SIGNED,
+  SILCTXT_MSG_PRIVATE_QUERY_UNKNOWN,
+  SILCTXT_MSG_PRIVATE_QUERY_FAILED,
+  SILCTXT_OWN_MSG_SIGNED,
+  SILCTXT_OWN_MSG_CHANNEL_SIGNED,
+  SILCTXT_NO_MSGS_GOT,
+  SILCTXT_NO_MSGS_SENT,
+  SILCTXT_OWN_MSG_PRIVATE_SIGNED,
+  SILCTXT_OWN_MSG_PRIVATE_QUERY_SIGNED,
 
   SILCTXT_FILL_2,
 
