@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcpkcs.c,v 1.44 2002/11/02 22:12:36 priikone Exp $ */
+/* $Id: silcpkcs.c,v 1.46.2.5 2002/12/05 21:38:17 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -620,6 +620,7 @@ SilcPublicKey silc_pkcs_public_key_alloc(const char *name,
   public_key->name = strdup(name);
   public_key->pk_len = pk_len;
   public_key->pk = silc_calloc(pk_len, sizeof(*public_key->pk));
+  public_key->pk_type = SILC_SKE_PK_TYPE_SILC;
   memcpy(public_key->pk, pk, pk_len);
 
   if (!silc_utf8_valid(identifier, strlen(identifier))) {
@@ -630,7 +631,7 @@ SilcPublicKey silc_pkcs_public_key_alloc(const char *name,
   }
 
   public_key->identifier = strdup(identifier);
-  public_key->len = 4 + 2 + strlen(name) + 2 + strlen(identifier) + pk_len;
+  public_key->len = 2 + strlen(name) + 2 + strlen(identifier) + pk_len;
   silc_free(tmp);
 
   return public_key;
@@ -686,7 +687,7 @@ silc_pkcs_public_key_encode(SilcPublicKey public_key, SilcUInt32 *len)
   SilcBuffer buf;
   unsigned char *ret;
 
-  buf = silc_buffer_alloc(public_key->len);
+  buf = silc_buffer_alloc(public_key->len + 4);
   silc_buffer_pull_tail(buf, SILC_BUFFER_END(buf));
 
   silc_buffer_format(buf,
@@ -699,7 +700,7 @@ silc_pkcs_public_key_encode(SilcPublicKey public_key, SilcUInt32 *len)
 					  public_key->pk_len),
 		     SILC_STR_END);
   if (len)
-    *len = public_key->len;
+    *len = public_key->len + 4;
 
   ret = silc_calloc(buf->len, sizeof(*ret));
   memcpy(ret, buf->data, buf->len);
@@ -719,8 +720,8 @@ silc_pkcs_public_key_data_encode(unsigned char *pk, SilcUInt32 pk_len,
   unsigned char *ret;
   SilcUInt32 totlen;
 
-  totlen = 4 + 2 + strlen(pkcs) + 2 + strlen(identifier) + pk_len;
-  buf = silc_buffer_alloc(totlen);
+  totlen = 2 + strlen(pkcs) + 2 + strlen(identifier) + pk_len;
+  buf = silc_buffer_alloc(totlen + 4);
   silc_buffer_pull_tail(buf, SILC_BUFFER_END(buf));
 
   silc_buffer_format(buf,
@@ -732,7 +733,7 @@ silc_pkcs_public_key_data_encode(unsigned char *pk, SilcUInt32 pk_len,
 		     SILC_STR_UI_XNSTRING(pk, pk_len),
 		     SILC_STR_END);
   if (len)
-    *len = totlen;
+    *len = totlen + 4;
 
   ret = silc_calloc(buf->len, sizeof(*ret));
   memcpy(ret, buf->data, buf->len);
@@ -767,7 +768,12 @@ bool silc_pkcs_public_key_decode(unsigned char *data, SilcUInt32 data_len,
     return FALSE;
   }
 
-  if (totlen != data_len) {
+#if 1 /* Backwards support, remove! */
+  if (totlen == data_len)
+    totlen -= 4;
+#endif
+
+  if (totlen + 4 != data_len) {
     silc_buffer_free(buf);
     return FALSE;
   }
@@ -826,6 +832,7 @@ bool silc_pkcs_public_key_decode(unsigned char *data, SilcUInt32 data_len,
     (*public_key)->identifier = ident;
     (*public_key)->pk = key_data;
     (*public_key)->pk_len = key_len;
+    (*public_key)->pk_type = SILC_SKE_PK_TYPE_SILC;
   }
 
   silc_buffer_free(buf);
@@ -842,6 +849,88 @@ bool silc_pkcs_public_key_decode(unsigned char *data, SilcUInt32 data_len,
   return FALSE;
 }
 
+/* Encodes Public Key Payload for transmitting public keys and certificates. */
+
+SilcBuffer silc_pkcs_public_key_payload_encode(SilcPublicKey public_key)
+{
+  SilcBuffer buffer;
+  unsigned char *pk;
+  SilcUInt32 pk_len;
+
+  if (!public_key)
+    return NULL;
+
+  pk = silc_pkcs_public_key_encode(public_key, &pk_len);
+  if (!pk)
+    return NULL;
+
+  buffer = silc_buffer_alloc_size(4 + pk_len);
+  if (!buffer) {
+    silc_free(pk);
+    return NULL;
+  }
+
+  silc_buffer_format(buffer,
+		     SILC_STR_UI_SHORT(pk_len),
+		     SILC_STR_UI_SHORT(public_key->pk_type),
+		     SILC_STR_UI_XNSTRING(pk, pk_len),
+		     SILC_STR_END);
+
+  silc_free(pk);
+  return buffer;
+}
+
+/* Decode Public Key Payload and decodes the public key inside it to
+   to `payload'. */
+
+bool silc_pkcs_public_key_payload_decode(unsigned char *data,
+					 SilcUInt32 data_len,
+					 SilcPublicKey *public_key)
+{
+  SilcBufferStruct buf;
+  SilcUInt16 pk_len, pk_type;
+  unsigned char *pk;
+  int ret;
+
+  if (!public_key)
+    return FALSE;
+
+#if 1
+  /* XXX 1.1 version support.  Check whether the data is actually raw
+     public key and attempt to decode.  Remove this later! */
+  if (silc_pkcs_public_key_decode(data, data_len, public_key)) {
+    (*public_key)->pk_type = SILC_SKE_PK_TYPE_SILC;
+    return TRUE;
+  }
+#endif
+
+  silc_buffer_set(&buf, data, data_len);
+  ret = silc_buffer_unformat(&buf,
+			     SILC_STR_UI_SHORT(&pk_len),
+			     SILC_STR_UI_SHORT(&pk_type),
+			     SILC_STR_END);
+  if (ret < 0 || pk_len > data_len - 4)
+    return FALSE;
+
+  /* For now we support only SILC public keys */
+  if (pk_type != SILC_SKE_PK_TYPE_SILC)
+    return FALSE;
+
+  silc_buffer_pull(&buf, 4);
+  ret = silc_buffer_unformat(&buf,
+			     SILC_STR_UI_XNSTRING(&pk, pk_len),
+			     SILC_STR_END);
+  silc_buffer_push(&buf, 4);
+  if (ret < 0)
+    return FALSE;
+
+  if (!silc_pkcs_public_key_decode(pk, pk_len, public_key))
+    return FALSE;
+  (*public_key)->pk_type = SILC_SKE_PK_TYPE_SILC;
+
+  return TRUE;
+}
+
 /* Compares two public keys and returns TRUE if they are same key, and
    FALSE if they are not same. */
 
@@ -876,6 +965,7 @@ SilcPublicKey silc_pkcs_public_key_copy(SilcPublicKey public_key)
 				strlen(public_key->identifier));
   key->pk = silc_memdup(public_key->pk, public_key->pk_len);
   key->pk_len = public_key->pk_len;
+  key->pk_type = public_key->pk_type;
 
   return key;
 }
@@ -1022,7 +1112,7 @@ bool silc_pkcs_private_key_decode(unsigned char *data, SilcUInt32 data_len,
 
 /* Internal routine to save public key */
 
-static bool silc_pkcs_save_public_key_internal(char *filename,
+static bool silc_pkcs_save_public_key_internal(const char *filename,
 					       unsigned char *data,
 					       SilcUInt32 data_len,
 					       SilcUInt32 encoding)
@@ -1062,7 +1152,7 @@ static bool silc_pkcs_save_public_key_internal(char *filename,
 
 /* Saves public key into file */
 
-bool silc_pkcs_save_public_key(char *filename, SilcPublicKey public_key,
+bool silc_pkcs_save_public_key(const char *filename, SilcPublicKey public_key,
 			       SilcUInt32 encoding)
 {
   unsigned char *data;
@@ -1078,9 +1168,8 @@ bool silc_pkcs_save_public_key(char *filename, SilcPublicKey public_key,
 
 /* Saves public key into file */
 
-bool silc_pkcs_save_public_key_data(char *filename, unsigned char *data,
-				    SilcUInt32 data_len,
-				    SilcUInt32 encoding)
+bool silc_pkcs_save_public_key_data(const char *filename, unsigned char *data,
+				    SilcUInt32 data_len, SilcUInt32 encoding)
 {
   return silc_pkcs_save_public_key_internal(filename, data, data_len,
 					    encoding);
@@ -1090,7 +1179,7 @@ bool silc_pkcs_save_public_key_data(char *filename, unsigned char *data,
 
 /* Internal routine to save private key. */
 
-static bool silc_pkcs_save_private_key_internal(char *filename,
+static bool silc_pkcs_save_private_key_internal(const char *filename,
 						unsigned char *data,
 						SilcUInt32 data_len,
 						unsigned char *key,
@@ -1098,11 +1187,11 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
 						SilcUInt32 encoding)
 {
   SilcCipher aes;
-  SilcHash md5;
+  SilcHash sha1;
   SilcHmac sha1hmac;
   SilcBuffer buf, enc;
-  SilcUInt32 len, blocklen;
-  unsigned char tmp[32], keymat[32];
+  SilcUInt32 len, blocklen, padlen;
+  unsigned char tmp[32], keymat[64];
   int i;
 
   memset(tmp, 0, sizeof(tmp));
@@ -1117,9 +1206,9 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
   if (blocklen * 2 > sizeof(tmp))
     return FALSE;
 
-  /* Allocate MD5 hash */
-  if (!silc_hash_alloc("md5", &md5)) {
-    SILC_LOG_ERROR(("Could not allocate MD5 hash, probably not registered"));
+  /* Allocate SHA1 hash */
+  if (!silc_hash_alloc("sha1", &sha1)) {
+    SILC_LOG_ERROR(("Could not allocate SHA1 hash, probably not registered"));
     silc_cipher_free(aes);
     return FALSE;
   }
@@ -1127,7 +1216,7 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
   /* Allocate HMAC */
   if (!silc_hmac_alloc("hmac-sha1-96", NULL, &sha1hmac)) {
     SILC_LOG_ERROR(("Could not allocate SHA1 HMAC, probably not registered"));
-    silc_hash_free(md5);
+    silc_hash_free(sha1);
     silc_cipher_free(aes);
     return FALSE;
   }
@@ -1136,33 +1225,33 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
      is 256 bits length, and derived by taking hash of the data, then 
      re-hashing the data and the previous digest, and using the first and
      second digest as the key. */
-  silc_hash_init(md5);
-  silc_hash_update(md5, key, key_len);
-  silc_hash_final(md5, keymat);
-  silc_hash_init(md5);
-  silc_hash_update(md5, key, key_len);
-  silc_hash_update(md5, keymat, 16);
-  silc_hash_final(md5, keymat + 16);
+  silc_hash_init(sha1);
+  silc_hash_update(sha1, key, key_len);
+  silc_hash_final(sha1, keymat);
+  silc_hash_init(sha1);
+  silc_hash_update(sha1, key, key_len);
+  silc_hash_update(sha1, keymat, 16);
+  silc_hash_final(sha1, keymat + 16);
 
   /* Set the key to the cipher */
-  silc_cipher_set_key(aes, keymat, sizeof(keymat) * 8);
+  silc_cipher_set_key(aes, keymat, 256);
 
   /* Encode the buffer to be encrypted.  Add padding to it too, at least
      block size of the cipher. */
 
   /* Allocate buffer for encryption */
   len = silc_hmac_len(sha1hmac);
-  enc = silc_buffer_alloc_size(data_len + 4 + 4 +
-			       (blocklen + (data_len % blocklen)) + len);
+  padlen = 16 + (16 - ((data_len + 4) % blocklen));
+  enc = silc_buffer_alloc_size(4 + 4 + data_len + padlen + len);
   if (!enc) {
     silc_hmac_free(sha1hmac);
-    silc_hash_free(md5);
+    silc_hash_free(sha1);
     silc_cipher_free(aes);
     return FALSE;
   }
 
   /* Generate padding */
-  for (i = 0; i < blocklen + (data_len % blocklen); i++)
+  for (i = 0; i < padlen; i++)
     tmp[i] = silc_rng_global_get_byte_fast();
 
   /* Put magic number */
@@ -1173,8 +1262,7 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
   silc_buffer_format(enc,
 		     SILC_STR_UI_INT(data_len),
 		     SILC_STR_UI_XNSTRING(data, data_len),
-		     SILC_STR_UI_XNSTRING(tmp, blocklen + (data_len %
-							   blocklen)),
+		     SILC_STR_UI_XNSTRING(tmp, padlen),
 		     SILC_STR_END);
 
   /* Encrypt. */
@@ -1196,7 +1284,7 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
   memset(keymat, 0, sizeof(keymat));
   memset(tmp, 0, sizeof(tmp));
   silc_hmac_free(sha1hmac);
-  silc_hash_free(md5);
+  silc_hash_free(sha1);
   silc_cipher_free(aes);
 
   data = enc->data;
@@ -1239,7 +1327,8 @@ static bool silc_pkcs_save_private_key_internal(char *filename,
 
 /* Saves private key into file. */
 
-bool silc_pkcs_save_private_key(char *filename, SilcPrivateKey private_key, 
+bool silc_pkcs_save_private_key(const char *filename,
+				SilcPrivateKey private_key,
 				unsigned char *passphrase,
 				SilcUInt32 passphrase_len,
 				SilcUInt32 encoding)
@@ -1260,12 +1349,16 @@ bool silc_pkcs_save_private_key(char *filename, SilcPrivateKey private_key,
 /* Loads public key from file and allocates new public key. Returns TRUE
    if loading was successful. */
 
-bool silc_pkcs_load_public_key(char *filename, SilcPublicKey *public_key,
+bool silc_pkcs_load_public_key(const char *filename, SilcPublicKey *public_key,
 			       SilcUInt32 encoding)
 {
   unsigned char *cp, *old, *data, byte;
   SilcUInt32 i, data_len, len;
 
+  SILC_LOG_DEBUG(("Loading public key `%s' with %s encoding", filename,
+		  encoding == SILC_PKCS_FILE_PEM ? "Base64" :
+		  encoding == SILC_PKCS_FILE_BIN ? "Binary" : "Unkonwn"));
+  
   old = data = silc_file_readfile(filename, &data_len);
   if (!data)
     return FALSE;
@@ -1313,22 +1406,184 @@ bool silc_pkcs_load_public_key(char *filename, SilcPublicKey *public_key,
   return TRUE;
 }
 
+#if 1 /* Old MD5 routine for private key loading, remove later! */
+struct MD5Context {
+  SilcUInt32 buf[4];
+  SilcUInt32 bits[2];
+  unsigned char in[64];
+};
+static void MD5Init(struct MD5Context *ctx)
+{
+  ctx->buf[0] = 0x67452301;
+  ctx->buf[1] = 0xefcdab89;
+  ctx->buf[2] = 0x98badcfe;
+  ctx->buf[3] = 0x10325476;
+  ctx->bits[0] = ctx->bits[1] = 0;
+}
+static void
+MD5Transform(SilcUInt32 buf[4], const unsigned char kbuf[64]);
+static void
+MD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned len)
+{
+  SilcUInt32 t;
+
+  t = ctx->bits[0];
+  if ((ctx->bits[0] = t + ((SilcUInt32)len << 3)) < t)
+    ctx->bits[1]++;	/* Carry from low to high */
+  ctx->bits[1] += (SilcUInt32)len >> 29;
+  t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
+  if ( t ) {
+    unsigned char *p = (unsigned char *)ctx->in + t;
+
+    t = 64-t;
+    if (len < t) {
+      memcpy(p, buf, len);
+      return;
+    }
+    memcpy(p, buf, t);
+    MD5Transform(ctx->buf, ctx->in);
+    buf += t;
+    len -= t;
+  }  while (len >= 64) {
+    memcpy(ctx->in, buf, 64);
+    MD5Transform(ctx->buf, ctx->in);
+    buf += 64;
+    len -= 64;
+  }
+  memcpy(ctx->in, buf, len);
+}
+
+static void MD5Final(unsigned char digest[16], struct MD5Context *ctx)
+{
+  unsigned count;
+  unsigned char *p;
+  count = (ctx->bits[0] >> 3) & 0x3F;
+  p = ctx->in + count;
+  *p++ = 0x80;
+  count = 64 - 1 - count;
+  if (count < 8) {
+    memset(p, 0, count);
+    MD5Transform(ctx->buf, ctx->in);
+    memset(ctx->in, 0, 56);
+  } else {
+    memset(p, 0, count-8);
+  }
+  ((SilcUInt32 *)ctx->in)[ 14 ] = ctx->bits[0];
+  ((SilcUInt32 *)ctx->in)[ 15 ] = ctx->bits[1];   
+  MD5Transform(ctx->buf, ctx->in);  
+  memcpy(digest, ctx->buf, 16);
+  memset(ctx, 0, sizeof(ctx));    /* In case it's sensitive */
+}
+#define F1(x, y, z) (z ^ (x & (y ^ z)))
+#define F2(x, y, z) F1(z, x, y)
+#define F3(x, y, z) (x ^ y ^ z)
+#define F4(x, y, z) (y ^ (x | ~z))
+#define MD5STEP(f, w, x, y, z, data, s) \
+	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s), w += x )
+static void
+MD5Transform(SilcUInt32 buf[4], const unsigned char kbuf[64])
+{
+  register SilcUInt32 a, b, c, d, i;
+  SilcUInt32 in[16];
+  for (i = 0; i < 16; i++)
+    SILC_GET32_MSB(in[i], kbuf + 4 * i);
+  a = buf[0];  b = buf[1];  c = buf[2];  d = buf[3];
+  MD5STEP(F1, a, b, c, d, in[ 0]+0xd76aa478,  7);
+  MD5STEP(F1, d, a, b, c, in[ 1]+0xe8c7b756, 12);
+  MD5STEP(F1, c, d, a, b, in[ 2]+0x242070db, 17);
+  MD5STEP(F1, b, c, d, a, in[ 3]+0xc1bdceee, 22);
+  MD5STEP(F1, a, b, c, d, in[ 4]+0xf57c0faf,  7);
+  MD5STEP(F1, d, a, b, c, in[ 5]+0x4787c62a, 12);
+  MD5STEP(F1, c, d, a, b, in[ 6]+0xa8304613, 17);
+  MD5STEP(F1, b, c, d, a, in[ 7]+0xfd469501, 22);
+  MD5STEP(F1, a, b, c, d, in[ 8]+0x698098d8,  7);
+  MD5STEP(F1, d, a, b, c, in[ 9]+0x8b44f7af, 12);
+  MD5STEP(F1, c, d, a, b, in[10]+0xffff5bb1, 17);
+  MD5STEP(F1, b, c, d, a, in[11]+0x895cd7be, 22);
+  MD5STEP(F1, a, b, c, d, in[12]+0x6b901122,  7);
+  MD5STEP(F1, d, a, b, c, in[13]+0xfd987193, 12);
+  MD5STEP(F1, c, d, a, b, in[14]+0xa679438e, 17);
+  MD5STEP(F1, b, c, d, a, in[15]+0x49b40821, 22);
+  MD5STEP(F2, a, b, c, d, in[ 1]+0xf61e2562,  5);
+  MD5STEP(F2, d, a, b, c, in[ 6]+0xc040b340,  9);
+  MD5STEP(F2, c, d, a, b, in[11]+0x265e5a51, 14);
+  MD5STEP(F2, b, c, d, a, in[ 0]+0xe9b6c7aa, 20);
+  MD5STEP(F2, a, b, c, d, in[ 5]+0xd62f105d,  5);
+  MD5STEP(F2, d, a, b, c, in[10]+0x02441453,  9);
+  MD5STEP(F2, c, d, a, b, in[15]+0xd8a1e681, 14);
+  MD5STEP(F2, b, c, d, a, in[ 4]+0xe7d3fbc8, 20);
+  MD5STEP(F2, a, b, c, d, in[ 9]+0x21e1cde6,  5);
+  MD5STEP(F2, d, a, b, c, in[14]+0xc33707d6,  9);
+  MD5STEP(F2, c, d, a, b, in[ 3]+0xf4d50d87, 14);
+  MD5STEP(F2, b, c, d, a, in[ 8]+0x455a14ed, 20);
+  MD5STEP(F2, a, b, c, d, in[13]+0xa9e3e905,  5);
+  MD5STEP(F2, d, a, b, c, in[ 2]+0xfcefa3f8,  9);
+  MD5STEP(F2, c, d, a, b, in[ 7]+0x676f02d9, 14);
+  MD5STEP(F2, b, c, d, a, in[12]+0x8d2a4c8a, 20);
+  MD5STEP(F3, a, b, c, d, in[ 5]+0xfffa3942,  4);
+  MD5STEP(F3, d, a, b, c, in[ 8]+0x8771f681, 11);
+  MD5STEP(F3, c, d, a, b, in[11]+0x6d9d6122, 16);
+  MD5STEP(F3, b, c, d, a, in[14]+0xfde5380c, 23);
+  MD5STEP(F3, a, b, c, d, in[ 1]+0xa4beea44,  4);
+  MD5STEP(F3, d, a, b, c, in[ 4]+0x4bdecfa9, 11);
+  MD5STEP(F3, c, d, a, b, in[ 7]+0xf6bb4b60, 16);
+  MD5STEP(F3, b, c, d, a, in[10]+0xbebfbc70, 23);
+  MD5STEP(F3, a, b, c, d, in[13]+0x289b7ec6,  4);
+  MD5STEP(F3, d, a, b, c, in[ 0]+0xeaa127fa, 11);
+  MD5STEP(F3, c, d, a, b, in[ 3]+0xd4ef3085, 16);
+  MD5STEP(F3, b, c, d, a, in[ 6]+0x04881d05, 23);
+  MD5STEP(F3, a, b, c, d, in[ 9]+0xd9d4d039,  4);
+  MD5STEP(F3, d, a, b, c, in[12]+0xe6db99e5, 11);
+  MD5STEP(F3, c, d, a, b, in[15]+0x1fa27cf8, 16);
+  MD5STEP(F3, b, c, d, a, in[ 2]+0xc4ac5665, 23);
+  MD5STEP(F4, a, b, c, d, in[ 0]+0xf4292244,  6);
+  MD5STEP(F4, d, a, b, c, in[ 7]+0x432aff97, 10);
+  MD5STEP(F4, c, d, a, b, in[14]+0xab9423a7, 15);
+  MD5STEP(F4, b, c, d, a, in[ 5]+0xfc93a039, 21);
+  MD5STEP(F4, a, b, c, d, in[12]+0x655b59c3,  6);
+  MD5STEP(F4, d, a, b, c, in[ 3]+0x8f0ccc92, 10);
+  MD5STEP(F4, c, d, a, b, in[10]+0xffeff47d, 15);
+  MD5STEP(F4, b, c, d, a, in[ 1]+0x85845dd1, 21);
+  MD5STEP(F4, a, b, c, d, in[ 8]+0x6fa87e4f,  6);
+  MD5STEP(F4, d, a, b, c, in[15]+0xfe2ce6e0, 10);
+  MD5STEP(F4, c, d, a, b, in[ 6]+0xa3014314, 15);
+  MD5STEP(F4, b, c, d, a, in[13]+0x4e0811a1, 21);
+  MD5STEP(F4, a, b, c, d, in[ 4]+0xf7537e82,  6);
+  MD5STEP(F4, d, a, b, c, in[11]+0xbd3af235, 10);
+  MD5STEP(F4, c, d, a, b, in[ 2]+0x2ad7d2bb, 15);
+  MD5STEP(F4, b, c, d, a, in[ 9]+0xeb86d391, 21);
+  buf[0] += a;  buf[1] += b;  buf[2] += c;  buf[3] += d;
+}
+#endif /* 1 */
+
 /* Load private key from file and allocates new private key. Returns TRUE
    if loading was successful. */
 
-bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
+bool silc_pkcs_load_private_key(const char *filename,
+				SilcPrivateKey *private_key,
 				unsigned char *passphrase,
 				SilcUInt32 passphrase_len,
 				SilcUInt32 encoding)
 {
   SilcCipher aes;
-  SilcHash md5;
+  SilcHash sha1;
   SilcHmac sha1hmac;
   SilcUInt32 blocklen;
-  unsigned char tmp[32], keymat[32];
+  unsigned char tmp[32], keymat[64];
   unsigned char *cp, *old, *data, byte;
   SilcUInt32 i, data_len, len, magic, mac_len;
 
+#if 1
+  /* Old support */
+  struct MD5Context md5;
+  bool oldsupport = FALSE;
+  int oldlen = 0;
+#endif /* 1 */
+
+  SILC_LOG_DEBUG(("Loading private key `%s' with %s encoding", filename,
+		  encoding == SILC_PKCS_FILE_PEM ? "Base64" :
+		  encoding == SILC_PKCS_FILE_BIN ? "Binary" : "Unkonwn"));
+  
   old = data = silc_file_readfile(filename, &data_len);
   if (!data)
     return FALSE;
@@ -1399,9 +1654,9 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
     return FALSE;
   }
 
-  /* Allocate MD5 hash */
-  if (!silc_hash_alloc("md5", &md5)) {
-    SILC_LOG_ERROR(("Could not allocate MD5 hash, probably not registered"));
+  /* Allocate SHA1 hash */
+  if (!silc_hash_alloc("sha1", &sha1)) {
+    SILC_LOG_ERROR(("Could not allocate SHA1 hash, probably not registered"));
     silc_cipher_free(aes);
     memset(old, 0, data_len);
     silc_free(old);
@@ -1411,7 +1666,7 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
   /* Allocate HMAC */
   if (!silc_hmac_alloc("hmac-sha1-96", NULL, &sha1hmac)) {
     SILC_LOG_ERROR(("Could not allocate SHA1 HMAC, probably not registered"));
-    silc_hash_free(md5);
+    silc_hash_free(sha1);
     silc_cipher_free(aes);
     memset(old, 0, data_len);
     silc_free(old);
@@ -1422,16 +1677,30 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
      is 256 bits length, and derived by taking hash of the data, then 
      re-hashing the data and the previous digest, and using the first and
      second digest as the key. */
-  silc_hash_init(md5);
-  silc_hash_update(md5, passphrase, passphrase_len);
-  silc_hash_final(md5, keymat);
-  silc_hash_init(md5);
-  silc_hash_update(md5, passphrase, passphrase_len);
-  silc_hash_update(md5, keymat, 16);
-  silc_hash_final(md5, keymat + 16);
+  silc_hash_init(sha1);
+  silc_hash_update(sha1, passphrase, passphrase_len);
+  silc_hash_final(sha1, keymat);
+  silc_hash_init(sha1);
+  silc_hash_update(sha1, passphrase, passphrase_len);
+  silc_hash_update(sha1, keymat, 16);
+  silc_hash_final(sha1, keymat + 16);
 
+#if 1
+ olds:
+  /* Old support */
+  if (oldsupport) {
+    MD5Init(&md5);
+    MD5Update(&md5, passphrase, passphrase_len);
+    MD5Final(keymat, &md5);
+    MD5Init(&md5);
+    MD5Update(&md5, passphrase, passphrase_len);
+    MD5Update(&md5, keymat, 16);
+    MD5Final(keymat + 16, &md5);
+  }
+#endif /* 1 */
+  
   /* Set the key to the cipher */
-  silc_cipher_set_key(aes, keymat, sizeof(keymat) * 8);
+  silc_cipher_set_key(aes, keymat, 256);
 
   /* First, verify the MAC of the private key data */
   mac_len = silc_hmac_len(sha1hmac);
@@ -1439,11 +1708,19 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
   silc_hmac_update(sha1hmac, data, len - mac_len);
   silc_hmac_final(sha1hmac, tmp, NULL);
   if (memcmp(tmp, data + (len - mac_len), mac_len)) {
+#if 1
+    /* Old support */
+    if (!oldsupport) {
+      oldsupport = TRUE;
+      goto olds;
+    }
+#endif
+    
     SILC_LOG_DEBUG(("Integrity check for private key failed"));
     memset(keymat, 0, sizeof(keymat));
     memset(tmp, 0, sizeof(tmp));
     silc_hmac_free(sha1hmac);
-    silc_hash_free(md5);
+    silc_hash_free(sha1);
     silc_cipher_free(aes);
     memset(old, 0, data_len);
     silc_free(old);
@@ -1452,15 +1729,20 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
   data += 4;
   len -= 4;
 
+#if 1
+  /* Old support */
+  oldlen = len - mac_len;
+#endif
+
   /* Decrypt the private key buffer */
-  silc_cipher_decrypt(aes, data, data, len - mac_len, silc_cipher_get_iv(aes));
+  silc_cipher_decrypt(aes, data, data, len - mac_len, NULL);
   SILC_GET32_MSB(i, data);
   if (i > len) {
     SILC_LOG_DEBUG(("Bad private key length in buffer!"));
     memset(keymat, 0, sizeof(keymat));
     memset(tmp, 0, sizeof(tmp));
     silc_hmac_free(sha1hmac);
-    silc_hash_free(md5);
+    silc_hash_free(sha1);
     silc_cipher_free(aes);
     memset(old, 0, data_len);
     silc_free(old);
@@ -1473,7 +1755,7 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
   memset(keymat, 0, sizeof(keymat));
   memset(tmp, 0, sizeof(tmp));
   silc_hmac_free(sha1hmac);
-  silc_hash_free(md5);
+  silc_hash_free(sha1);
   silc_cipher_free(aes);
 
   /* Now decode the actual private key */
@@ -1483,6 +1765,13 @@ bool silc_pkcs_load_private_key(char *filename, SilcPrivateKey *private_key,
     return FALSE;
   }
 
+#if 1
+  /* Old support */
+  if (oldsupport || (oldlen & 15))
+    silc_pkcs_save_private_key((char *)filename, *private_key,
+			       passphrase, passphrase_len, encoding);
+#endif
+    
   memset(old, 0, data_len);
   silc_free(old);
   return TRUE;
